@ECHO OFF

SET ARCH=%PROCESSOR_ARCHITECTURE%

2>NUL CALL :CASE_%ARCH% # jump to arch
IF ERRORLEVEL 1 CALL :DEFAULT_CASE

:CASE_AMD64
    <NUL set /p="windows amd64"
    GOTO END_CASE
:DEFAULT_CASE
    EXIT 1
:END_CASE
    EXIT 0